/* -*- C++ -*-
 *
 * This file is a part of LEMON, a generic C++ optimization library
 *
 * Copyright (C) 2003-2006
 * Egervary Jeno Kombinatorikus Optimalizalasi Kutatocsoport
 * (Egervary Research Group on Combinatorial Optimization, EGRES).
 *
 * Permission to use, modify and distribute this software is granted
 * provided that this copyright notice appears in all copies. For
 * precise terms see the accompanying LICENSE file.
 *
 * This software is provided "AS IS" with no warranty of any kind,
 * express or implied, and with no claim as to its suitability for any
 * purpose.
 *
 */

#include<iostream>
#include<lemon/concept_check.h>

#include<lemon/smart_graph.h>
#include<lemon/concepts/graph.h>
#include<lemon/concepts/ugraph.h>
#include<lemon/concepts/bpugraph.h>

#include<lemon/list_graph.h>
#include<lemon/full_graph.h>
#include<lemon/graph_adaptor.h>
#include<lemon/ugraph_adaptor.h>
#include<lemon/bpugraph_adaptor.h>

#include"test/test_tools.h"
#include"test/graph_test.h"

/**
\file
This test makes consistency checks of graph adaptors.

\todo More extensive tests are needed 
*/

using namespace lemon;
using namespace lemon::concepts;



int main() 
{
  {
    typedef Graph Graph;
    checkConcept<Graph, GraphAdaptor<Graph> >();

    checkConcept<Graph, RevGraphAdaptor<Graph> >();

    checkConcept<Graph, SubGraphAdaptor<Graph, 
      Graph::NodeMap<bool> , Graph::EdgeMap<bool> > >();
    checkConcept<Graph, NodeSubGraphAdaptor<Graph, 
      Graph::NodeMap<bool> > >();
    checkConcept<Graph, EdgeSubGraphAdaptor<Graph, 
      Graph::EdgeMap<bool> > >();
    
    checkConcept<Graph, ResGraphAdaptor<Graph, int, 
      Graph::EdgeMap<int>, Graph::EdgeMap<int> > >();

    checkConcept<Graph, ErasingFirstGraphAdaptor<Graph, 
      Graph::NodeMap<Graph::Edge> > >(); 

    checkConcept<UGraph, UndirGraphAdaptor<Graph> >();

    checkConcept<UGraph, SubUGraphAdaptor<UGraph, 
      UGraph::NodeMap<bool> , UGraph::UEdgeMap<bool> > >();
    checkConcept<UGraph, NodeSubUGraphAdaptor<UGraph, 
      UGraph::NodeMap<bool> > >();
    checkConcept<UGraph, EdgeSubUGraphAdaptor<UGraph, 
      UGraph::UEdgeMap<bool> > >();

    checkConcept<Graph, DirUGraphAdaptor<UGraph, 
      UGraph::UEdgeMap<bool> > >();

    checkConcept<BpUGraph, BpUGraphAdaptor<BpUGraph> >();

    checkConcept<BpUGraph, SwapBpUGraphAdaptor<BpUGraph> >();

  }
  std::cout << __FILE__ ": All tests passed.\n";

  return 0;
}
